#include "Eigen/Dense"
#include <iostream>
#include <cassert>

template<typename T, int M>
T Givens(Eigen::Matrix<T,M,1> a, Eigen::Matrix<T,M,1> y) {
  auto m=a.size();
  for (auto i=m-2;i>=0;i--) {
    Eigen::Matrix<T,M,M> G=Eigen::Matrix<T,M,M>::Identity(m,m);
    T norm_a_tilde=a.block(i,0,2,1).norm();
    y(i)=(a(i)*y(i)+a(i+1)*y(i+1))/norm_a_tilde;
    y(i+1)=(-a(i+1)*y(i)+a(i)*y(i+1))/norm_a_tilde;
    a(i)=norm_a_tilde; a(i+1)=0;
  }
  return y(0)/a(0);
}

template<typename T, int M>
T dEdp(const T& p, const Eigen::Matrix<T,M,1> &x, Eigen::Matrix<T,M,1> &y) {
  return p*x.dot(x)-x.dot(y);
}

template<typename T, int M>
T ddEdpp(const Eigen::Matrix<T,M,1> &x) {
  return x.dot(x);
}

int main(int argc, char* argv[]) {
  assert(argc==2); auto M=std::stoi(argv[1]);
  using T=float; 
  using VT=Eigen::Matrix<T,Eigen::Dynamic,1>;
  VT x=VT::Random(M), y=VT::Random(M); 
  x=x.cwiseProduct(x);
  std::cout << "x^T=" << x.transpose() << std::endl
            << "y^T=" << y.transpose() << std::endl;
  T p=Givens(x,y);
  std::cout << "p=" << p << std::endl;
  std::cout << "E'=" << dEdp(p,x,y) << std::endl;
  std::cout << "E''=" << ddEdpp(x) << std::endl;
  return 0;
}
