#include "Eigen/Dense"
#include <iostream>
#include <cassert>

template<typename T, int M>
T Householder(Eigen::Matrix<T,M,1> x, Eigen::Matrix<T,M,1> y) {
  using VT=Eigen::Matrix<T,M,1>; using MT=Eigen::Matrix<T,M,M>;
  auto m=x.size();
  VT v=x+x(0)/fabs(x(0))*x.norm()*VT::Unit(m,0);
  MT H=MT::Identity(m,m)-2*v*v.transpose()/v.dot(v);
  x=H*x; y=H*y;
  return y(0)/x(0);
}

template<typename T, int M>
T dEdp(const T& p, const Eigen::Matrix<T,M,1> &x, Eigen::Matrix<T,M,1> &y) {
  return p*x.dot(x)-x.dot(y);
}

template<typename T, int M>
T ddEdpp(const Eigen::Matrix<T,M,1> &x) {
  return x.dot(x);
}

int main(int argc, char* argv[]) {
  assert(argc==2); auto M=std::stoi(argv[1]);
  using T=float; 
  using VT=Eigen::Matrix<T,Eigen::Dynamic,1>;
  VT x=VT::Random(M), y=VT::Random(M); 
  x=x.cwiseProduct(x);
  std::cout << "x^T=" << x.transpose() << std::endl
            << "y^T=" << y.transpose() << std::endl;
  T p=Householder(x,y);
  std::cout << "p=" << p << std::endl;
  std::cout << "E'=" << dEdp(p,x,y) << std::endl;
  std::cout << "E''=" << ddEdpp(x) << std::endl;
  return 0;
}
