#include "Eigen/Dense"
#include <iostream>
#include <cassert>
#include <fstream>

template<typename T, int M>
T NormalEquation(const Eigen::Matrix<T,M,1> &x, Eigen::Matrix<T,M,1> &y) {
  return y.dot(x)/x.dot(x);
}

template<typename T, int M>
T dEdp(const T& p, const Eigen::Matrix<T,M,1> &x, Eigen::Matrix<T,M,1> &y) {
  return p*x.dot(x)-x.dot(y);
}

template<typename T, int M>
T ddEdpp(const Eigen::Matrix<T,M,1> &x) {
  return x.dot(x);
}

int main(int argc, char* argv[]) {
  assert(argc==2); auto M=std::stoi(argv[1]);
  using T=float; 
  using VT=Eigen::Matrix<T,Eigen::Dynamic,1>;
  VT x=VT::Random(M), y=VT::Random(M); 
  x=x.cwiseProduct(x);
  std::cout << "x^T=" << x.transpose() << std::endl
            << "y^T=" << y.transpose() << std::endl;

  T p=NormalEquation(x,y);
  std::cout << "p=" << p << std::endl;
  std::cout << "E'=" << dEdp(p,x,y) << std::endl;
  std::cout << "E''=" << ddEdpp(x) << std::endl;

  std::ofstream out("data");
  for (auto i=0;i<M;i++) out << x(i) << " " << y(i)<< std::endl;
  return 0;
}
